/*  @(#)56  1.7 src/com/tivoli/pd/jazn/demo/PDJaasDemo.java, pd.jazn, am611 07/03/15 14:13:25 @(#)  */

/*
 *
 *
 * Licensed Materials - Property of IBM
 * 5748-XX8
 * (c) Copyright International Business Machines Corp. 2003
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * File Name: PDJaasDemo.java
 */


import com.tivoli.pd.jazn.*;
import com.tivoli.pd.jutil.PDException;

import java.io.*;
import java.net.*;
import java.security.*;

import javax.security.auth.*;
import javax.security.auth.callback.*;
import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;

/**
 * IBM Tivoli Access Manager for e-business PDJaasDemo class.
 *
 * <p>
 * This class demonstrates the use of the Java Authentication and Authorization
 * Service related to Access Manager classes.
 * <p>
 * To use the demo you must have a configuration file.  This file is the same
 * as the one specified to the com.tivoli.pd.jcfg.SvrSslCfg
 * configuration program.
 * <p>
 * The demo will create a PDAuthorizationContext using the configuraton file.
 * The PDAuthorizationContext represents the connection to the Access Manager
 * secure domain and the servers therein. A PDLoginModule is then created. The
 * PDAuthorizationContext is set as default authorization context for the
 * PDLoginModule.
 * <p>
 * The demo will then repeatedly prompt for whether or not a password is
 * required.  The answer to the prompt determines the type of login context the
 * demo is to use.
 * A LoginContext is then created with the correct login context type and a
 * PDCallbackHandler. The login function of the LoginContext is called. The
 * PDCallbackHandler, based on login context type, will request the user id and
 * optionally the password. After a successful login, the demo requests the
 * resource and actions to test for the logged in Subject. Each test will
 * result in a true or false result which indicates the user's authorization to
 * perform the specified actions on the specified resource.
 * <p>
 * You may quit the demo by specifying "quit" at any of the prompts except the
 * user id and password prompts since these prompts are within the
 * PDCallbackHandler.
 */

public final class PDJaasDemo
{

    private final String PDJaasDemo_java_sourceCodeID="$Id: @(#)56  1.7 src/com/tivoli/pd/jazn/demo/PDJaasDemo.java, pd.jazn, am611 07/03/15 14:13:25 @(#) $";
    public static final String exitString = "quit";
    public static final String configFilePrompt = "configuration file";
    public static final String pwdReqdPrompt = "password required";
    public static final String pwdReqdDefault = "Y";
    public static final String resourcePrompt = "resource";
    public static final String resourceDefault = "/";
    public static final String actionsPrompt = "actions";
    public static final String actionsDefault = "T";

    private static String configFile_ = null;
    private static String pwdReqd_ = null;
    private static String resource_ = null;
    private static String actions_ = null;
    boolean exit_ = false;

    /**
     * Prompts for input.
     *
     * <p>
     * @param reader  the reader used to retrieve input.
     *
     * @param prompt  string used for prompt when retrieving input.
     *
     * @param def     string default returned when no input is provided.
     *
     * <p>
     * @return        string input.  This is the value specified by the default
     *                string when no input is provided.
     */
    private String prompt(BufferedReader reader,
                          String prompt,
                          String def) throws java.io.IOException
    {
        String result = null;
        System.out.print(prompt);

        if (def != null)
        {
            System.out.print(" [");
            System.out.print(def);
            System.out.print("]");
        }

        System.out.print(":  ");
        result = reader.readLine();
        if (result.length() == 0)
        {
            result = def;
        }
        return result;
    }

    /**
     * Constructs a PDAuthorizationContext based on configuration file input.
     * This method will continue to request a valid configuration file until
     * either a PDAuthorizationContext can be successfully constructed or
     * the demo is exited.
     *
     * @param reader  the reader used to retrieve the configuration file
     *                input.  The configuration file specified is used
     *                to construct a PDAuthorizationContext.
     *
     * @return        PDAuthorizationContext based on configuration file
     *                specified.
     */
    private PDAuthorizationContext getAuthorizationContext(BufferedReader reader)
    {
        PDAuthorizationContext ctxt=null;
        while ((!exit()) && (ctxt == null))
        {
            try
            {
                configFile_ = prompt(reader, configFilePrompt, configFile_);
                if (!exit(configFile_))
                {
                    File cFile = new File(configFile_);
                    if (cFile.exists())
                    {
                        if (cFile.isFile())
                        {
                            URL configURL = new URL("file:///" + configFile_);
                            ctxt = new PDAuthorizationContext(configURL);
                        }
                        else
                        {
                            System.out.println("Specified configuration file is not a file.");
                        }
                    }
                    else
                    {
                        System.out.println("Specified configuration file does not exist.");
                    }
                }
            }
            catch (Exception e)
            {
                e.printStackTrace();
                ctxt = null;
            }
        }
        return ctxt;
    }

    /**
     * Prompts for whether or not to require the password associated with the user id.
     *
     * <p>
     * @param reader  the reader used to retrieve input.
     *
     * <p>
     * @return        string input.  This is the value specified by the private pwdReqd_
     *                string when no input is provided.
     */
    private String readRequired(BufferedReader reader)
    {
        boolean notValid = true;
        while ((!exit()) && (notValid))
        {
            try
            {
                pwdReqd_ = prompt(reader, pwdReqdPrompt, pwdReqd_);
                if (!exit(pwdReqd_))
                {
                    notValid = false;
                }
            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }
        return pwdReqd_;
    }

    /**
     * Constructs a PDPermission object for the specified resource and actions.
     * The PDPermission object is the combination of a resource and actions.
     * The resource is a string which identifies the accessed resource.
     * The actions are a string that identifies the modes of access.
     * <p>
     * This method will continue to request a valid resource and valid
     * actions until either a PDPermission can be successfully constructed
     * or the demo is exited.
     *
     * @param reader           the reader used to retrieve the resource and
     *                         actions input.  The resource and actions
     *                         specified are used to construct a
     *                         PDPermission.
     *
     * @return                 PDPermission based on resource and actions
     *                         specified.
     */
    private PDPermission getPermission(BufferedReader reader)
    {
        PDPermission permission = null;
        while ((!exit()) && (permission == null))
        {
            try
            {
                resource_ = prompt(reader, resourcePrompt, resource_);
                if (!exit(resource_))
                {
                    actions_ = prompt(reader, actionsPrompt, actions_);
                    if (!exit(actions_))
                    {
                        permission = new PDPermission(resource_, actions_);
                    }
                }
            }
            catch (Exception e)
            {
                e.printStackTrace();
                permission = null;
            }
        }
        return permission;
    }

    /**
     * Constructs a PDJaasDemo object.
     */
    public PDJaasDemo(String [] argv)
    {
        pwdReqd_ = pwdReqdDefault;
        resource_ = resourceDefault;
        actions_ = actionsDefault;

        if (argv.length > 0)
        {
            configFile_ = argv[0];
            if (argv.length > 1)
            {
                pwdReqd_ = argv[1];
                if (argv.length > 2)
                {
                    resource_ = argv[3];
                    if (argv.length > 3)
                    {
                        actions_ = argv[4];
                    }
                }
            }
        }
    }

    /**
     * Returns a boolean to indicate the demo should exit.
     */
    public boolean exit()
    {
        return exit_;
    }

    /**
     * Tests an input string to determine if the demo should exit.
     *
     * @param input  string to test.  A case insensitive comparison is done
     *               against the string "quit" to determine whether the
     *               the demo should exit.
     *
     * @return       boolean to indicate the demo should exit.
     */
    private boolean exit(String input)
    {
        if (input.equalsIgnoreCase(exitString))
        {
            exit_ = true;
        }
        else
        {
            exit_ = false;
        }
        return exit_;
    }

    /**
     * Main program.
     */
    public static void main (String [] argv)
    {
        PDCallbackHandler handler = null;
        String loginContextType = null;
        LoginContext lc = null;
        Subject user = null;
        boolean noError = true;

        // construct demo object.
        PDJaasDemo demo = new PDJaasDemo(argv);

        // construct reader from standard input stream.
        InputStreamReader input = new InputStreamReader(System.in);
        BufferedReader reader = new BufferedReader(input);

        // construct authorization context based on input configuration file.
        PDAuthorizationContext ctxt = demo.getAuthorizationContext(reader);

        // while no indication to exit
        while (!demo.exit())
        {
            try
            {
                // determine whether to require a password or not
                String pwdReqd = demo.readRequired(reader);
                if (!demo.exit())
                {
                    // password is not required
                    if (pwdReqd.equalsIgnoreCase("n")) {
                        loginContextType = "pd-nopass";
                    }
                    // password is required
                    else {
                        loginContextType = "pd";
                    }

                    // first time through, create a PDCallbackHandler object.
                    if (handler == null) {
                        handler = new PDCallbackHandler();
                    }

                    try {

                        // create an instance of the PDLoginModule
                        PDLoginModule lm = new PDLoginModule();

                        // set the default Authorization Context for the PDLoginModule
                        lm.setDefaultAuthorizationContext(ctxt);

                        // create an instance of a LoginContext. The first parameter
                        // determines whether a password is required or not. The second
                        // parameter is an instance of the PDCallbackHandler
                        lc = new LoginContext(loginContextType, handler);

                        // attempt to login
                        lc.login();

                        // get the subject after a successful login
                        user = lc.getSubject();

                        // indicate login was successful
                        noError = true;

                    }
                    catch (LoginException e)
                    {
                        // indicate login failed
                        noError = false;
                        System.out.println("LoginException occurred with information provided: " + e);
                    }

                    // if login was successful
                    if (noError) {

                        // create a PDPersmission object based on user input
                        final PDPermission permission = demo.getPermission(reader);
                        if (!demo.exit())
                        {
                            try
                            {
                                // become the logged in user
                                Subject.doAs(user,
                                    (new PrivilegedExceptionAction()
                                    {
                                        public Object run() throws Exception
                                        {
                                            // check for permission
                                            boolean access=permission.implies(permission);
                                            if (!access)
                                            {
                                                PDException pde = permission.getPDException();
                                                if (pde != null)
                                                {
                                                    throw pde;
                                                }
                                            }
                                            System.out.println(access);
                                            return null;
                                        }
                                    }
                                    ));
                            }
                            catch (PrivilegedActionException pae)
                            {
                                PDException pde = (PDException)pae.getException();
                                throw pde;
                            }
                        }
                    }
                }
            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }

        // close the context
        if (ctxt != null)
        {
            try
            {
               ctxt.close();
            }
            catch (PDException pde)
            {
                pde.printStackTrace();
            }
        }
    }

}

/**
 * Access Manager PDCallbackHandler class.
 *
 * <p>
 * This class is the JAAS authentication callback handler for the PDJaasDemo.
 *
 * <p>
 * This handler supports the callbacks required by the PDJaasDemo. This
 * handler is used during authentication by providing the required callbacks.
 * These required callbacks are the javax.security.auth.callback.NameCallback
 * and the javax.security.auth.callback.PasswordCallback.
 *
 * <p>
 * The javax.security.auth.callback.NameCallback callback prompts the user for
 * a user id to use during authentication.
 *
 * <p>
 * The javax.security.auth.callback.PasswordCallback callback prompts the user
 * for a password to use during authentication.
 *
 * <p>
 * The PDLoginModule uses the PDCallbackHandler to prompt the user for the
 * authentication information required based on the login context type defined
 * for the LoginContext.
 */
class PDCallbackHandler implements CallbackHandler {

    private String userID;
    private String password;
    private BufferedReader rdr;

    /**
     * Constructs the handler
     */
    public PDCallbackHandler() {
            this.userID = null;
            this.password = null;
            this.rdr = new BufferedReader (new InputStreamReader(System.in));
    }

    /**
     * Constructs the handler with a user id and password.
     *
     * @param userID ID representing authenticating user.
     * @param password Private password of the user.
     */
    public PDCallbackHandler(String userID, String password) {
            this.userID = userID;
            this.password = password;
    }

    /**
     * Constructs the handler with a user id only.
     *
     * @param userID ID representing authenticating user.
     */
    public PDCallbackHandler(String userID) {
            this.userID = userID;
    }

    /**
     * Handles the required callbacks.
     *
     * @param callbacks Callback[] of authentication callbacks to be
     *         handled.
     *
     * @exception UnsupportedCallbackException Thrown if a callback is passed
     *             to the handler it does not support.
     */
    public void handle(Callback[] callbacks) throws IOException,UnsupportedCallbackException {

        for (int i = 0; i < callbacks.length; i++) {

            Callback cbk = callbacks[i];

            // NameCallback routine
            if (cbk instanceof NameCallback) {

                NameCallback nc = (NameCallback)cbk;
                // request the userid
                String user = prompt("user", this.userID);
                this.userID = user;
                nc.setName(this.userID);

            }
            // PasswordCallback routine
            else if (cbk instanceof PasswordCallback) {

                PasswordCallback pc = (PasswordCallback)cbk;
                // request the password
                String pwd = prompt("password", this.password);
                this.password = pwd;
                pc.setPassword(this.password.toCharArray());

            }
            // Any other type of Callback is not supported
            else {
                throw new UnsupportedCallbackException(callbacks[i], "Unrecognized Callback");
            }
        }

    }

    /**
     * Prompts for input.
     *
     * @param prompt  string used for prompt when retrieving input.
     * @param def     string default returned when no input is provided.
     *
     * @return        string input.  This is the value specified by the default
     *                string when no input is provided.
     */
    private String prompt(String prompt, String def) throws java.io.IOException
    {
        String result = null;
        System.out.print(prompt);

        if (def != null)
        {
            System.out.print(" [");
            System.out.print(def);
            System.out.print("]");
        }

        System.out.print(":  ");
        result = this.rdr.readLine();
        if (result.length() == 0)
        {
            result = def;
        }
        return result;
    }

}
